/* IMS F006A example program
   Copyright INMOS Limited 1991 */

/* Simple Ethernet listener. This program initialises the IMS B431
   Ethernet interface in promiscuous receive mode and prints out the
   header segment of every packet it receives. It also periodically
   requests and displays accumulated Ethernet statistics as gathered
   by the IMS B431 device driver. */

#include <time.h>
#include <misc.h>
#include <stdio.h>
#include <stdlib.h>
#include <b431io.h>
#include <process.h>

/* A function to continuously request Ethernet statistics,
   once every 20 seconds, this uses the channel to the IMS B431
   device driver and will be executed as a transputer process */

void request_stats(
	Process	*p,
	Channel *to_b431 )
{
	p = p;	/* Gets rid of compiler warning message */

	while ( 1 )
	{
		ProcWait( (int) CLOCKS_PER_SEC * 20 );
		B431_Ether_Stats( to_b431 );
	}
}

/* Print the header segment of a packet */

void print_packet_header(
	unsigned char	*ethernet_packet,
	int	ethernet_packet_length )
{
	int	i;

	printf( "Packet received, length = %d bytes\n[ ",
		ethernet_packet_length );

	for ( i = 0; i < PACKET_HEADER_SIZE; i++ )
		printf( "0x%02x ", ethernet_packet[i] );
	printf( "]\n" );
}

/* print the Ethernet statistics structure */

void print_ether_stats( ETHER_STATS ether_stats )
{
	printf( "Ethernet statistics:\n" );
	printf( "tx_packets = %lu, ", ether_stats.tx_packets );
	printf( "rx_packets = %lu\n", ether_stats.rx_packets );
	printf( "framing_errors = %lu, ", ether_stats.framing_errors );
	printf( "crc_errors = %lu\n", ether_stats.crc_errors );
	printf( "packets_dropped = %lu, ", ether_stats.packets_dropped );
	printf( "packets_missed = %lu\n", ether_stats.packets_missed );
	printf( "deferred_transmits = %lu, ", ether_stats.deferred_transmits) ;
	printf( "late_collisions = %lu, ", ether_stats.late_collisions );
	printf( "carrier_lost = %lu\n", ether_stats.carrier_lost );
	printf( "failed_retries = %lu, ", ether_stats.no_more_retries );
	printf( "single_retries = %lu, ", ether_stats.single_retries );
	printf( "many_retries = %lu\n", ether_stats.multiple_retries );
	printf( "average_tdr_value = %lu\n", ether_stats.average_tdr_value );
}

/* Initialise the Ethernet interface and then start it running.
   Continuously call B431_Waitfor_Event() to receive ethernet packets
   and print their header. Start a parallel process to request Ethernet
   statistics and print them */

int main()
{
	Process	*p;
	char	result;
	ETHER_STATS	ether_stats;
	/* Arbitrary physical address */
	unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE] =
		{ 0x22, 0x44, 0x88, 0x88, 0x44, 0x22 };
	/* Logical address filter disables multicast packet reception */
	unsigned char	logical_address_filter[LOGICAL_ADDRESS_FILTER_SIZE] =
		{ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 };
	int	error_code, receive_packet_length;	
	unsigned char	receive_packet[MAX_PACKET_LENGTH];
	unsigned char	failed_packet_data[FAILED_PACKET_LENGTH];
	Channel	*to_b431, *from_b431, *cancel = ChanAlloc();

	/* Channels to / from IMS B431 device driver are obtained from
	   the configuration environment */

	to_b431 = (Channel *) get_param( 4 );
	from_b431 = (Channel *) get_param( 3 );

	printf( "IMS-F006 Ethernet listener\n" );

	/* Initialise and start the Ethernet interface in promiscuous
	   packet receive mode */

	result = B431_Init_Normal( from_b431, to_b431,
			physical_address, logical_address_filter,
			MEMORY_CHECK | PROMISCUOUS_RX );
	if ( result != INIT_SUCCESS )
	{
		printf( "B431_Init_Normal() failed = %d\n", result );
		exit_terminate(0);
	}

	result = B431_Start_Ether( from_b431, to_b431 );
	if ( result != START_SUCCESS )
	{
		printf( "B431_Start_Ether() failed = %d\n", result );
		exit_terminate(0);
	}

	/* Allocate and start a parallel process to request Ethernet
	   statistics once every 20 seconds. It is passed a single parameter,
	   the channel to the B431 device driver */

	p = ProcAlloc( ( void (*)() ) request_stats, 0, 1, to_b431 ) ;
	ProcRun( p );

	/* Wait for Ethernet interface activity. Print Ethernet statistics
	   or the headers of packets received. Ignore everything else */

	while ( 1 )
	{
		result = B431_Waitfor_Event( from_b431, cancel,
				&ether_stats,
				receive_packet, &receive_packet_length,
				&error_code, failed_packet_data );

		switch (result)
		{
			case B431_ETHER_STATS:
				print_ether_stats( ether_stats );
				break;

			case B431_RX_PACKET:
				print_packet_header( receive_packet,
							receive_packet_length );
				break;
		}
	}
}
