/*
-- Copyright INMOS Limited 1991
--
-- **************************************************************************
--
-- Group:	Iq software
--
-- Product:	IMS-F006 Ethernet TRAM support software
--
-- File:	b431test.c
--
-- Version:	1.0
--
-- **************************************************************************
--
-- B431 device driver test library. This file contains procedures for
-- performing diagnostics tests on an IMS-B431 or IMS-B407 ethernet TRAM
-- when executing the ethernet TRAM device driver (b431drvr). 
--
-- **************************************************************************
--
-- History:	
--
-- (25-SEP-91) Bob Wipfel, Created file.
--
-- **************************************************************************
*/

#include <stdlib.h>

#include <time.h>
#include <process.h>
#include <channel.h>

#include "b431io.h"
#include "b431test.h"

/* Maximum number of bytes sent in test packets. This does not include
   the packet header segment and trailing CRC bytes. */

#define TEST_DATA_SIZE ( MAX_LOOPBACK_LENGTH - (PACKET_HEADER_SIZE + 4) )

static void supply(
	Process *p,
	Channel *c )
{
	p = p;

	ChanOutChar( c, (char) 1 );
}


static void consume(
	Process *p,
	Channel *c )
{
	char	token;

	p = p;

	token = ChanInChar( c );
}


static void handshake(
	Channel	*cancel,
	Channel *cancel_handshake )
{
	Process	*s = ProcAlloc( supply, 0, 1, cancel ),
		*c = ProcAlloc( consume, 0, 1, cancel_handshake );

	if ( ( s == NULL ) || ( c == NULL ) ) abort();

	ProcPar( s, c, NULL );

	ProcAllocClean( s );
	ProcAllocClean( c );
}


static void timeout(
	Process *p,
	Channel *cancel,
	Channel	*cancel_handshake )
{
	p = p;

	switch ( ProcTimerAlt( ProcTimePlus( ProcTime(),
		(int) CLOCKS_PER_SEC ), cancel_handshake, NULL ) )
	{
		case -1:
		{
			handshake( cancel, cancel_handshake );
			break;
		}
			
		case 0:
		{
			char	token;

			token = ChanInChar( cancel_handshake );
			ChanOutChar( cancel, token );
			break;
		}
	}
}


static void event(
	Process	*p,
	Channel	*cancel,
	Channel *cancel_handshake,
	int	*result,
	Channel	*from_b431,
	ETHER_STATS	*ethernet_stats,
	unsigned char	*receive_packet,
	int	*receive_packet_length,
	int	*error_code,
	unsigned char	*failed_packet_data )
{
	p = p;

	*result = B431_Waitfor_Event( from_b431, cancel,
		ethernet_stats, receive_packet, receive_packet_length,
		error_code, failed_packet_data );

	switch ( *result )
	{
		case B431_RX_PACKET:
		case B431_ERROR_REPORT:

			handshake( cancel_handshake, cancel );
			break;

		case B431_WAIT_CANCELLED:

			ChanOutChar( cancel_handshake, (char) 1 );
			break;
	}
}


static void stop(
	Channel	*from_b431,
	Channel *to_b431 )
{
	Channel		cancel;
	ETHER_STATS	ethernet_stats;
	int		error_code, receive_packet_length;
	unsigned char	receive_packet[MAX_PACKET_LENGTH],
			failed_packet_data[FAILED_PACKET_LENGTH];

	ChanInit( &cancel );

	B431_Stop_Ether( to_b431 );	/* If active */
	
	while ( B431_Waitfor_Event( from_b431, &cancel, &ethernet_stats,
		receive_packet, &receive_packet_length,
		&error_code, failed_packet_data ) != B431_STOP_ETHER )
		;
}


static int loop( 
	Channel		*from_b431,
	Channel		*to_b431,
	const unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE],
	const int	test_data_length,
	int		*error_code )
{
	Process		*t, *e;
	Channel		cancel, cancel_handshake;
	ETHER_STATS	ethernet_stats;
	int		result, receive_packet_length, i;
	unsigned char	test_data[TEST_DATA_SIZE],
			receive_packet[MAX_PACKET_LENGTH],
			failed_packet_data[FAILED_PACKET_LENGTH];

	ChanInit( &cancel );
	ChanInit( &cancel_handshake );

	for ( i = 0; i < test_data_length; i++ )
		test_data[i] = (unsigned char) (test_data_length - i);

	B431_Tx_Packet2( to_b431, physical_address, 0,
				test_data, test_data_length ); 

	t = ProcAlloc( timeout, 0, 2, &cancel, &cancel_handshake );
	e = ProcAlloc( event, 0, 9, &cancel, &cancel_handshake, &result,
		from_b431, &ethernet_stats, receive_packet, 
		&receive_packet_length, error_code, failed_packet_data );

	if ( ( t == NULL ) || ( e == NULL ) ) abort();

	ProcPar( t, e, NULL );

	ProcAllocClean( t );
	ProcAllocClean( e );

	switch ( result )
	{
		case B431_ERROR_REPORT:
			return( TEST_HARDWARE_ERROR );

		case B431_WAIT_CANCELLED:
			return( TEST_LOOPBACK_FAILED );
	}

	receive_packet_length -= 4; /* Remove trailing CRC bytes */

	if ( receive_packet_length != (test_data_length + PACKET_HEADER_SIZE) )
		return( TEST_LOOPBACK_FAILED );

	for ( i = 0; i < PHYSICAL_ADDRESS_SIZE; i++ )
		if ( receive_packet[i] != physical_address[i] )
			return( TEST_LOOPBACK_FAILED );

	for ( i = 0; i < test_data_length; i++ )
		if ( test_data[i] != receive_packet[i + PACKET_HEADER_SIZE] )
			return( TEST_LOOPBACK_FAILED );

	return( TEST_PASSED );
}


int B431_Internal_Loopback( 
	Channel		*from_b431,
	Channel		*to_b431,
	const unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE],
	int		*error_code )
{
	int	result, test_data_length;

	stop( from_b431, to_b431 );

	switch ( B431_Init_Loopback( from_b431, to_b431,
		physical_address, MEMORY_CHECK | INTERNAL_LOOPBACK ) )
	{
		case INIT_MEMORY_FAULT:
			return( TEST_MEMORY_FAULT );

		case INIT_HARDWARE_FAILED:
			return( TEST_INIT_FAILED );
	}

	(void) B431_Start_Ether( from_b431, to_b431 );

	for ( test_data_length = 0;
		test_data_length <= TEST_DATA_SIZE; test_data_length++ )
	{
		result = loop( from_b431, to_b431, physical_address,
			test_data_length, error_code );

		if ( result == TEST_PASSED )
			continue;

		stop( from_b431, to_b431 );
		return( result );
	}

	stop( from_b431, to_b431 );
	return( TEST_PASSED );
}


int B431_External_Loopback( 
	Channel		*from_b431,
	Channel		*to_b431,
	const unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE],
	int		*error_code )
{
	int	result, test_data_length;

	stop( from_b431, to_b431 );

	switch ( B431_Init_Loopback( from_b431, to_b431,
		physical_address, MEMORY_CHECK ) )
	{
		case INIT_MEMORY_FAULT:
			return( TEST_MEMORY_FAULT );

		case INIT_HARDWARE_FAILED:
			return( TEST_INIT_FAILED );
	}

	(void) B431_Start_Ether( from_b431, to_b431 );

	for ( test_data_length = 0;
		test_data_length <= TEST_DATA_SIZE; test_data_length++ )
	{
		int retry_count;

		result = TEST_LOOPBACK_FAILED;

		for ( retry_count = 0;
			( retry_count < 32 ) & ( result != TEST_PASSED );
			retry_count++ )
		{
			result = loop( from_b431, to_b431, physical_address,
				test_data_length, error_code );

			if ( result == TEST_HARDWARE_ERROR )
			{
				stop( from_b431, to_b431 );
				return( result );
			}
		}

		if ( result == TEST_LOOPBACK_FAILED )
		{
			stop( from_b431, to_b431 );
			return( result );
		}
	}

	stop( from_b431, to_b431 );
	return( TEST_PASSED );
}
